"""
Constant values used in the project

Variables:
    VERSION           - The version of the program as a string
    BASE_API_URL      - The root URL of the API
    USER_AGENT        - The default user-agent to send with every request
    METADATA_FILENAME - The filename for metadata files saved in jars
"""
# Static values
VERSION = "1.3.0"
BASE_API_URL = "https://api.spiget.org/v2"
USER_AGENT = f"Spud/{VERSION}"
METADATA_FILENAME = ".spud_meta.json"
